#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Copyright 2016 Eddie Antonio Santos <easantos@ualberta.ca>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

NEGATIVE_INFINITY = -float('inf')


def modification_time(filename):
    """
    Returns modification time of the given filename.
    Returns negative infinity if the file does not exist.
    """
    if not os.path.exists(filename):
        return NEGATIVE_INFINITY
    return os.stat(filename).st_mtime



#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Copyright 2016 Eddie Antonio Santos <easantos@ualberta.ca>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import shutil
import sys
import warnings
import codecs

from paperpal import __version__ as version
from paperpal.k2pdfopt_wrapper import k2pdfopt
from paperpal.modification_time import modification_time
from paperpal.zotero import Zotero
from paperpal.fix_bibliography import fix_bibliography


parser = argparse.ArgumentParser(description="utilities for a streamlined "
                                             "bibliography workflow")
parser.add_argument('-v', '--version',
                    action='version',
                    version='%(prog)s ' + version)
subparsers = parser.add_subparsers(title='subcommands',
                                   description='valid subcommands',
                                   dest='command',
                                   help='')

# export options.
export_parser = subparsers.add_parser('export',
                                      help='exports a collection as a BibTeX '
                                           'bibliography')
export_parser.add_argument('collection',
                           help='name of the collection to export')
export_parser.add_argument('destination',
                           nargs='?',
                           default='bibliography.bib',
                           help='name of exported file '
                                '(default "bibliography.bib")')
export_parser.add_argument('-c', '--clean',
                           action='store_true',
                           help='Exports as ASCII, removes URLs, and ensures '
                           'each entry with a date also has a year field. '
                           'Same as running `paperpal fix-bibliography` on '
                           'the output')
export_parser.set_defaults(translator='bibtex')
translator_group = export_parser.add_mutually_exclusive_group()
translator_group.add_argument('--bibtex',
                              action='store_const', const='bibtex',
                              dest='translator',
                              help='use BibTeX as the translator '
                                   '(default)')
translator_group.add_argument('--biblatex',
                              action='store_const', const='biblatex',
                              dest='translator',
                              help='use BibLaTeX as the translator')
translator_group.add_argument('--better-bibtex',
                              action='store_const', const='better-bibtex',
                              dest='translator',
                              help='use Better BibTeX as the translator')
translator_group.add_argument('--better-biblatex',
                              action='store_const', const='better-biblatex',
                              dest='translator',
                              help='use Better BibLaTeX as the translator')

# copy options.
copy_parser = subparsers.add_parser('copy',
                                    help='copies PDFs from the collection '
                                         'to a directory')
copy_parser.add_argument('collection',
                         help='name of the collection')
copy_parser.add_argument('directory',
                         help='destination of copied PDFs')

# to-ebook options.
ebook_parser = subparsers.add_parser('to-ebook',
                                     help='copies PDFs from the collection '
                                          'to a directory')
ebook_parser.add_argument('collection',
                          help='name of the collection')
ebook_parser.add_argument('directory',
                          help='destination of translated PDFs')
ebook_parser.add_argument('args',
                          metavar='...',
                          nargs=argparse.REMAINDER,
                          help='remaining arguments are passed to k2pdfopt '
                               'unchanged')

# copy options.
fix_parser = subparsers.add_parser('fix-bibliography',
                                   help='Fixes an existing .bib file for '
                                   'easier usage with ieeetran.cls and '
                                   'sig-alternate.cls')
fix_parser.add_argument('bibliography',
                        help='name of input file')
fix_parser.add_argument('destination',
                        default='bibliography.bib',
                        help='name of exported file '
                        '(default "bibliography.bib")')


def export_bibliography(collection, destination, fix_bib=False,
                        *args, **kwargs):
    bib_string = Zotero().export_bibliography(collection, *args, **kwargs)
                  
    if fix_bib:
        bib_string = fix_bibliography(bib_string)

    bib_unicode = bib_string.decode('UTF-8')
    with codecs.open(destination, encoding='UTF-8', mode='w') as output_file:
        output_file.write(bib_unicode)


def copy_pdfs(collection, directory):
    for source, destination, _ in pdfs_to_update(collection, directory):
        shutil.copy(source, destination)


def to_ebook(collection, directory, *k2pdfopt_args):
    pdfs = pdfs_to_update(collection, directory)
    for source, destination, info in pdfs:
        k2pdfopt(source,
                 destination,
                 author=authors_to_string(*info['authors']),
                 title=info['title'],
                 *k2pdfopt_args)


def author_to_string(author):
    """
    >>> author_to_string({'first': 'Devender'})
    'Devender'
    >>> author_to_string({'first': 'M. Night', 'last': 'Shamylaan'})
    'M. Night Shamylaan'
    """
    names = []
    if author.get('first', None):
        names.append(author['first'])
    if author.get('last', None):
        names.append(author['last'])

    return ' '.join(names)


def authors_to_string(*authors):
    """
    >>> author1 = {'first': 'S.', 'last': 'Miyamoto'}
    >>> author2 = {'first': 'K.', 'last': 'Kondo'}
    >>> author3 = {'first': 'H.', 'last': 'Tanaka'}
    >>> authors_to_string(author1)
    'S. Miyamoto'
    >>> authors_to_string(author1, author2)
    'S. Miyamoto and K. Kondo'
    >>> authors_to_string(author1, author2, author3)
    'S. Miyamoto, K. Kondo, H. Tanaka'
    """
    # Two authors: use " and "
    if len(authors) == 2:
        return (author_to_string(authors[0]) + ' and ' +
                author_to_string(authors[1]))

    # Use a comma separated list
    return ', '.join(author_to_string(auth) for auth in authors)


def pdfs_to_update(collection, destination_directory, with_info=False):
    """
    Yields a tuple of PDFs that should be updated. The tuple is the original
    pdf, and the new PDF path. If with_info is requested,
    """
    pdfs = Zotero().list_papers(collection)

    def destination(*paths):
        return os.path.join(destination_directory, *paths)

    for item in pdfs:
        cite_key = item['cite_key']
        pdf_path = item['pdf_filename']

        if cite_key is None:
            warnings.warn('Please generate a cite key for '
                          '{author} "{title}" {year}'.format(**item))
            continue
        if pdf_path is None:
            warnings.warn('No PDF found for {cite_key}'.format(**item))
            continue

        new_pdf = destination(cite_key + '.pdf')
        if not os.path.exists(pdf_path):
            warnings.warn("I can't make sense of this path: "
                          '{!r}. Skipping...'.format(pdf_path))
            continue

        if modification_time(pdf_path) > modification_time(new_pdf):
            yield pdf_path, new_pdf, item


def fix_bibliography_wrapper(source, destination):
    with codecs.open(source, encoding="UTF-8") as bibtex_file:
        result = fix_bibliography(bibtex_file.read())

    with codecs.open(destination, encoding='UTF-8', mode='w') as output_file:
        output_file.write(result)


def main():
    options = parser.parse_args()

    if options.command == 'export':
        return export_bibliography(options.collection,
                                   options.destination,
                                   fix_bib=options.clean,
                                   translator=options.translator)
    elif options.command == 'copy':
        return copy_pdfs(options.collection, options.directory)
    elif options.command == 'to-ebook':
        return to_ebook(options.collection, options.directory,
                        *options.args)
    elif options.command == 'fix-bibliography':
        return fix_bibliography_wrapper(options.bibliography,
                                        options.destination)


if __name__ == '__main__':
    exit(main())



#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Copyright 2016 Eddie Antonio Santos <easantos@ualberta.ca>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
import tempfile
import urllib

import mozrepl

from .resource_path import resource_path


with open(resource_path('zotero-bridge.js')) as javascript_file:
    JS_RUNTIME = javascript_file.read()

NAME_TO_TRANSLATOR_ID = {
    'bibtex':           '9cb70025-a888-4a29-a210-93ec52da40d4',
    'biblatex':         'b6e39b57-8942-4d11-8259-342c46ce395f',
    'better-bibtex':    'f895aa0d-f28e-47fe-b247-2ea77c6ed583',
    'better-biblatex':  'f895aa0d-f28e-47fe-b247-2ea77c6ed583'

}


class ZoteroError(Exception):
    def __init__(self, reason):
        super(ZoteroError, self).__init__(repr(reason))
        self.reason = reason


class Zotero(object):
    def __init__(self):
        self.repl = mozrepl.Mozrepl()

    def list_papers(self, collection):
        result = self._send_to_repl('list', collection=collection)
        return [fix_filename(item) for item in result]

    def export_bibliography(self, collection, translator=None):
        if translator is not None:
            translator = NAME_TO_TRANSLATOR_ID[translator]

        with tempfile.NamedTemporaryFile() as temporary:
            self._send_to_repl('exportBibliography',
                               filename=temporary.name,
                               collection=collection,
                               translator=translator)
            # Return the contents.
            temporary.seek(0)
            return temporary.read()

    def _send_to_repl(self, action, **options):
        # INTENTIONALLY inject code into the code snippet.
        code = JS_RUNTIME % {
            'action': json.dumps(action),
            'options': json.dumps(options),
            'open_comment': '/*',
            'close_comment': '*/'
        }

        result = self.repl.execute(code)
        status, json_payload = tuple(result)
        payload = json.loads(json_payload)

        if status == 'error':
            raise ZoteroError(payload)

        return payload


def fix_filename(item):
    """
    >>> datum = {'pdf_filename': 'file:///Users/Foo%20Bar/herp.pdf'}
    >>> fix_filename(datum)['pdf_filename']
    '/Users/Foo Bar/herp.pdf'
    """
    if item['pdf_filename'] is None:
        return item

    original = item['pdf_filename']
    if original.startswith('file://'):
        original = original[len('file://'):]
    filename = urllib.unquote(original)
    item.update(pdf_filename=filename)

    return item



#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Copyright 2016 Eddie Antonio Santos <easantos@ualberta.ca>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess

__all__ = ['k2pdfopt']


def k2pdfopt(filename, destination, *extra_args, **kwargs):
    args = ['k2pdfopt',
            '-ui-', '-x',
            '-mode', '2col',
            '-dev', 'kpw']

    if 'author' in kwargs:
        args += ['-author', kwargs['author']]
    if 'title' in kwargs:
        args += ['-title', kwargs['title']]

    args += extra_args
    args += ['-o', destination, filename]

    return subprocess.call(args)



from setuptools import setup, find_packages

from paperpal import __version__ as version


def slurp(filename):
    with open(filename) as opened_file:
        return opened_file.read()


setup(name='paperpal',
      version=version,
      description='Paper management with Zotero',
      long_description=slurp('README.rst'),
      url='http://github.com/eddieantonio/paperpal',
      entry_points = {
          'console_scripts': ['paperpal=paperpal.__main__:main'],
      },
      author='Eddie Antonio Santos',
      author_email='easantos@ualberta.ca',
      install_requires=slurp('./requirements.txt').split('\n')[:-1],
      license='Apache 2.0',
      packages=['paperpal'],
      package_data={'paperpal': ['*.js']},
      classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Text Processing :: Indexing',
      ]
)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import re

import bibtexparser

from bibtexparser.bparser import BibTexParser
from bibtexparser.customization import homogeneize_latex_encoding


__all__ = ['fix_bibliography']


def fix_bibliography(bibtex_string):
    """
    Given a bibliography file, `fixes` it by removing URLs from articles,
    ASCIIifying all the fields and replacing dates with years.
    """

    # Make a parser that will ASCIIify everything:
    # See: https://bibtexparser.readthedocs.io/en/v0.6.2/tutorial.html#accents-and-weird-characters
    parser = BibTexParser()
    parser.customization = homogeneize_latex_encoding

    bibtex = bibtexparser.loads(bibtex_string, parser=parser)

    for entry in bibtex.entries:
        fix_entry(entry)

    # TODO: if py3k, do not encode.
    return bibtexparser.dumps(bibtex).encode("UTF-8")


def fix_entry(entry):
    # Add the year from the date.
    if 'date' in entry:
        entry['year'] = str(parse_year(entry['date']))

    # In most entry types, get rid of the URL.
    if entry['ENTRYTYPE'] not in ('online', 'misc') and 'url' in entry:
        del entry['url']

    # Get rid of the DOI.
    if 'doi' in entry:
        del entry['doi']


def parse_year(string):
    """
    >>> parse_year('2013')
    2013
    >>> parse_year('April 2014')
    2014
    >>> parse_year('06/2012')
    2012
    """

    match = re.search('\d{4}', string)
    if not match:
        raise ValueError('Could not parse year in ' + repr(string))

    return int(match.group(0))
